classdef NumericPanel < handle
    %NUMERICPANEL Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        numvarlist
        panelhandle
        layoutstructure;
 
        eventlistener;
    end
    
    methods
        function obj = NumericPanel(parent , numvarlist, embed, varargin)

           obj.numvarlist = numvarlist; 
           obj.panelhandle = uipanel(parent, 'Unit' , 'Pixels' , 'BackgroundColor' , [0.95 0.95 0.95] , varargin{:});
           set(obj.panelhandle,'DeleteFcn' , @(o,e) obj.destructor() , 'ResizeFcn' , @(o,e) obj.onResize(o));
           
           if (embed)
               pos = get(parent , 'Position');
               set(obj.panelhandle ,'Position' ,[0 0 pos(3) pos(4)]);
           end
           
           obj.eventlistener{2} = numvarlist.addlistener( 'structureChanged', @(o,ev) obj.structureChanged(numvarlist)  );
           
           obj.organizePanel(numvarlist);
           
           
           if (embed)
              set(obj.panelhandle , 'Units' , 'normalize'); 
              NumericOptionMenu( parent , numvarlist);
           
           end

           
           
           
        end
        
        function structureChanged(obj,numvarlist)
            printc(mfilename, 'structureChanged');
            delete(allchild(obj.panelhandle));
            obj.layoutstructure.mainnode.destructor();
            obj.layoutstructure = struct();
            printc(mfilename , 'reorganizePanel');
            
            
            units = get(obj.panelhandle , 'Units');
            set(obj.panelhandle ,'Units' , 'Pixels');
            obj.organizePanel(numvarlist);
            set(obj.panelhandle  ,'Units' , units);
        end


        function onResize(obj,handle)
            units = get(handle , 'Units');
            set(handle,'Units' , 'Pixels');
            pos = get(handle, 'Position');
            if (~isempty(pos)) 
                obj.layoutstructure.mainnode.makeLayoutHappen( get(handle, 'Position'));
            end
            set(handle,'Units' , units);
        end


        function organizePanel(obj , numvarlist)
            printc(mfilename , 'organizePanel');
            mainnode = LayoutNode(-1 , -1 , 'vertical');

            selected = numvarlist.getAllowed();
            
            miscpanel  = Panel(obj.panelhandle , 0 ,  0 , 'BackgroundColor' , [0.95 0.95 0.95] );
            for i = 1:length(selected)
                
                sectionnode = LayoutNode( 0 , 0 , 'vertical');
                

                if (numvarlist.containsSubVars(selected{i}))
                    keys = numvarlist.getNumVarList( selected{i} );
                    
                    if (~isempty(keys))
                    sectionnode.setWeights( 4*length(keys) , 1);
                    panel = Panel(obj.panelhandle , 14 ,  0 , 'BackgroundColor' , [0.95 0.95 0.95] , 'Title', selected{i});
                    sectionnode.addGUIobject(1,1,panel);
                    obj.layoutstructure.panels.(selected{i}) = panel;
                    
                    
                    for j = 1:length(keys)
                        
                        if (numvarlist.isFieldEnabled(selected{i} ,keys{j}))
                            subnode = LayoutNode( 2 , 1);
                            subnode.addHandle( 1, 1, uicontrol( panel.handle  , 'Style' , 'text' , 'Units', 'Pixels' , 'BackgroundColor' , [0.95 0.95 0.95], 'HorizontalAlignment' , 'left' , ...
                        'FontSize', 12, 'String' ,  keys{j} ), 'minsize' , [Inf,20]);
                             handle = uicontrol( panel.handle  , 'Style' , 'text' , 'Units', 'Pixels' , 'BackgroundColor' , [0.95 0.95 0.95], 'HorizontalAlignment' , 'left' , ...
                        'FontSize', 12, 'String' , '');
                             subnode.addHandle( 1, 1, handle, 'minsize' , [Inf,20]); 

                            numvarlist.setNumHandle(selected{i} , keys{j} , handle );
                            panel.mainnode.addNode(subnode);
                            empty = false;
                        end
                    end
                    mainnode.addNode(sectionnode);
                    
                    end

                else
                        sectionnode.setWeights(  2 , 1);
                        
                        subnode = LayoutNode( 2 , 1);
                        
                        subnode.addHandle( 1, 1, uicontrol( miscpanel.handle  , 'Style' , 'text' , 'Units', 'Pixels' , 'BackgroundColor' , [0.95 0.95 0.95], 'HorizontalAlignment' , 'left' , ...
                    'FontSize', 12, 'String' ,  selected{i} ), 'minsize' , [Inf,20]);
                         handle = uicontrol( miscpanel.handle , 'Style' , 'text' , 'Units', 'Pixels' , 'BackgroundColor' , [0.95 0.95 0.95], 'HorizontalAlignment' , 'left' , ...
                    'FontSize', 12, 'String' , '');    
                        
                        subnode.addHandle(1,1, handle, 'minsize' , [Inf,20]);
                        sectionnode.addNode(subnode);
                        miscpanel.mainnode.addNode(sectionnode);
                        numvarlist.setNumHandle_s(selected{i} , handle);
                end
                obj.layoutstructure.(selected{i}) = sectionnode;
            end
            printc(mfilename , 'components placed');
            %fixme: memory leak mogelijk
            miscs = length( miscpanel.mainnode.handles);
            obj.layoutstructure.misc = LayoutNode(miscs*2 , 1);
            obj.layoutstructure.misc.setHandle( miscpanel.handle , 'minsize' ,[0,0]);
            if (miscs > 0), mainnode.addNode( obj.layoutstructure.misc  ); end
            
            printc(mfilename, 'prepare layout');
            [w,h] = mainnode.getPrefSize();
            mainnode.setOptions( 'Add' , 1 , 'sliderthreshold' , [200 h] , 'panel' , obj.panelhandle);
            
            mainnode.makeLayoutHappen(  get(obj.panelhandle , 'Position')    );
            obj.layoutstructure.mainnode = mainnode;
            printc(mfilename, 'layout done');

        end
        function destructor(obj)
            global sOutput
            %D disp(['Destructor: ' mfilename]);
            %if (exist('sOutput','var')), sOutput.switchOffNumeric(); end;  %FIXME
            
            obj.numvarlist.clearHandles();
            obj.layoutstructure.mainnode.destructor();
            for i=1:length(obj.eventlistener)
               delete(obj.eventlistener{i});
            end
            delete(obj);
        end
        
    end
    
end

