classdef Plot3D
    %PLOT3D Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        axeshandle
        xmap
        ymap
        zmap
        color
        plotops = {};
    end
     
    methods
        function obj = Plot3D(axeshandle, color, xmap , ymap, zmap)
            obj.axeshandle = axeshandle;
            obj.xmap = xmap;
            obj.ymap = ymap;
            obj.zmap = zmap;
            
            obj.color = color;
        end

        function outputPoint(obj,  xout, hout, fout , i , plotops, pointmsg)
            it = i(end);            
            x = obj.xmap(xout, hout, fout, it);
            y = obj.ymap(xout, hout, fout, it);
            z = obj.zmap(xout, hout, fout, it);
            
	    pointmsg = [num2str(i) ': ' pointmsg];
            line([[] x],[[] y],[[] z], 'EraseMode','none' , 'Parent' , obj.axeshandle ,'Marker' , 'o'  , 'ButtonDownFcn' , @(o,e) disp(pointmsg), plotops{:});
            
            d = axis(obj.axeshandle);
            skew = 0.01*[d(2)-d(1) d(4)-d(3) d(6)-d(5)];            
            
            text(x+skew(1),y+skew(2),z+skew(3), num2str(i) , 'Parent' ,obj.axeshandle, 'ButtonDownFcn' , @(o,e) disp(pointmsg));
        end        
        
        function output(obj, session , xout, s , hout, fout , i)
            
            it = i(end);
            if i(end)>1, jj = 1;else jj = 0;end

            
            x0 = obj.xmap(xout, hout, fout , it-jj);
            x1 = obj.xmap(xout, hout, fout, it);
            y0 = obj.ymap(xout, hout, fout , it-jj);
            y1 = obj.ymap(xout, hout, fout, it);
            z0 = obj.zmap(xout, hout, fout , it-jj);
            z1 = obj.zmap(xout, hout, fout, it);            
 
            line([x0 x1] , [y0 y1], [z0 z1] , 'EraseMode','none' , 'Parent' , obj.axeshandle , 'Color' , obj.color  , obj.plotops{:});
            
            
	    if ((it > 2) && ( s.index == (it - 1) ))
                x = obj.xmap(xout, hout, fout , it-1);
                y = obj.ymap(xout, hout, fout , it-1);
                z = obj.zmap(xout, hout, fout , it-1);

                pointselector = PlotPointSelector(session);
                curve = session.getCurrentCurve();
                
                line([[] x],[[] y],[[] z] , 'Linestyle','*','Color','r', 'Parent', obj.axeshandle  ,'UserData' ,  [0 0 0 0 0 0] , 'ButtonDownFcn' ,@(o,e) pointselector.selectPointByIndex(o , curve,s.index)  );
                d = axis(obj.axeshandle);
                skew = 0.01*[d(2)-d(1) d(4)-d(3) d(6)-d(5)];
                text(x+skew(1), y+skew(2),z+skew(3),  s.label , 'Parent' , obj.axeshandle ,'UserData' ,  [0 0 0 0 0 0] , 'ButtonDownFcn' ,@(o,e) pointselector.selectPointByIndex(o , curve,s.index)  );
            end
       %     hold off
        end
        
    end
    methods(Static)
        function plotcurve(axeshandle, xmap , ymap , zmap, curve , pointselector , varargin)
            len = length(curve.x);
            x_ax = zeros(1,len);
            y_ax = x_ax;
            z_ax = x_ax;
            for i=1:len
               x_ax(i) = xmap(curve.x , curve.h , curve.f , i);
               y_ax(i) = ymap(curve.x , curve.h , curve.f , i);
               z_ax(i) = zmap(curve.x , curve.h , curve.f , i);
            end

            color = curve.getCurveType().getDefaultColor();
	    axes(axeshandle);
	    hold all
            plot3(axeshandle, x_ax , y_ax , z_ax, 'Color' , color,  varargin{:});
            
            d = axis(axeshandle);
            skew = 0.01*[d(2)-d(1) d(4)-d(3) d(6)-d(5)];
            for i = 2:(length(curve.s)-1)
               s_x = xmap(curve.x , curve.h , curve.f , curve.s(i).index );
               s_y = ymap(curve.x , curve.h , curve.f , curve.s(i).index );
               s_z = zmap(curve.x , curve.h , curve.f , curve.s(i).index );
               pointselectopts = {}; 
	       if (~isempty(pointselector)), pointselectopts = {'UserData' , [0 0 0 0 0 0] , 'ButtonDownFcn' ,@(o,e) pointselector.selectPoint(o , curve,i)}; end
	       line([[] s_x],[[] s_y], [[] s_z], 'Marker','*','Color','r', 'Parent', axeshandle, pointselectopts{:}); 
               text(s_x+skew(1), s_y+skew(2), s_z+skew(3), curve.s(i).label , 'Parent' , axeshandle, pointselectopts{:}); 
            end
            
        end
        
    end    
    
end
