classdef Session < handle
    %SESSION Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
    systemspath
	system;
        
        currentcurve;
        continuedata
        starterdata
        optioninterface
       
        numericdata
        windowmanager;
        curvemanager;
        branchmanager;
        plotmanager;
        
        unlocked = 1;

        sessiondata;
        configmanager;
    end
    
    events
       stateChanged       
       pointChanged
       curveChanged  
       newSystem
    end
    
    
    methods
        %%
        function obj = Session(systemspath)
            if (nargin == 1)
               obj.systemspath = systemspath; 
            else
               obj.systemspath = '../Systems/';
            end
            
            
            obj.system = [];
            obj.currentcurve = [];
            obj.curvemanager = [];

            obj.continuedata = ContDataModel();
            obj.starterdata = StartDataModel(obj);
            obj.optioninterface = ContinuerOptionInterface( obj.continuedata ,  obj.starterdata );
            
            obj.branchmanager = BranchManager(obj);
            obj.plotmanager = PlotManager(obj);
            
            obj.numericdata = NumericVarList(obj);
            
            obj.sessiondata =  SessionData([obj.systemspath  '/Session.mat']);
            obj.sessiondata.restoreState(obj);
            obj.configmanager = ConfigManager(obj.sessiondata);
            obj.windowmanager = WindowManager(obj , obj.sessiondata);
        end

        function reSet(obj)
            
            obj.system = [];
            obj.currentcurve = [];
            obj.curvemanager = [];            
            obj.notify('pointChanged');
            obj.notify('curveChanged');
            obj.notify('newSystem');
            obj.notify('stateChanged');
            
        end
        
        %%
        function setCurveType(obj , curvedescr)
            ct = obj.getCurveType();
            if ~strcmp(curvedescr.getLabel() , ct.getLabel())
                obj.getCurrentCurve().setCurveType(curvedescr);
                obj.warnObservers('curveChanged');
            end
        end
        
        function ct =  getCurveType(obj)
            ct = obj.getCurrentCurve().getCurveType();
        end
        
        function setPointType(obj, pointdescr)
            pt = obj.getPointType();
            
            if ~strcmp(pointdescr.getLabel() , pt.getLabel())
                obj.getCurrentCurve().setPointType( pointdescr);
                obj.branchmanager.setup(obj);
                obj.notify('stateChanged');
            end
        end
        %%

        function sys = getSystem(obj)
		if(~isempty(obj.system))
		    sys = obj.system;
		else
		    sys = System('EMPTY'); %FIXME memory leak
		end
        end
        
        function b = hasSystem(obj)
            b = ~isempty(obj.system);
        end

        function pt = getPointType(obj)
           pt = obj.getCurrentCurve().getPointType(); 
        end
        
        
        function changeSystem(obj,system)
            if (isempty(obj.system) || ~system.cmp(obj.system))
               obj.system = system;
               obj.warnObservers('newSystem');
            end
        end        
        
        function changeCurveManager(obj,system,curvemanager)
            obj.changeSystem(system);
            if(isempty(obj.curvemanager) || ~obj.curvemanager.cmp(curvemanager))
               obj.curvemanager = curvemanager;
               %obj.warnObservers('xChanged');
            end
            
        end
        function loadNewCurve(obj , system, curvemanager)
            obj.changeCurveManager(system,curvemanager);
            obj.currentcurve = curvemanager.getBlankCurve();
            obj.branchmanager.setup(obj); %point changed 
            obj.warnObservers('curveChanged');
        end
        
        function changeCurve(obj,system,curvemanager,curve , startdata, contdata)
            obj.changeCurveManager(system, curvemanager);
            
            obj.currentcurve = curve;
            
            obj.branchmanager.setup(obj); %point changed 
            
            obj.warnObservers('curveChanged'); %FIXME risque
           
            if (~isempty(contdata)),  obj.continuedata.copyOver(contdata); end 
            if (~isempty(startdata)), obj.starterdata.copyOver(startdata); end 
            
            
            
        end
        
        function loadInPoint(obj, system , curvemanager, parentcurve , selectedpoint , pointdata , startdata, contdata)
            parameters = pointdata.parameters;
            coordinates = pointdata.coordinates;
            
            for i = 1:length(parameters(:,1))
               startdata.setParameter( parameters{i,1} , parameters{i,2} ); 
            end
            for i = 1:length(coordinates(:,1))
               startdata.setCoordinate( coordinates{i,1} , coordinates{i,2} ); 
            end  
            curve = curvemanager.getBlankCurve(selectedpoint);
            curve.globals = parentcurve.globals;
            curve.depth = parentcurve.depth+1;
            curve.setParentCT( parentcurve.getCurveType().getLabel());
            
            obj.changeCurve(system,curvemanager, curve , startdata, contdata);

            ptype = deblank(selectedpoint.label);
            pt = StaticPointType.getPointTypeObject(ptype);

            if ~isempty(pt)
                 obj.currentcurve.setPointType( pt );
                 obj.branchmanager.setup(obj);
            end
            
            
        end
        
        
        function warnObservers(obj , state)
            notify(obj , state);
            if ~strcmp(state , 'stateChanged')
                notify(obj,'stateChanged');
            end
        end
        
        function sd = getStartData(obj)
           sd = obj.starterdata; 
        end
        function contdata = getContData(obj)
           contdata = obj.continuedata;
        end
        
        function wm = getWindowManager(obj)
           wm = obj.windowmanager; 
        end
        function bm = getBranchManager(obj)
            bm = obj.branchmanager;
        end
	function cm = getCurveManager(obj)
		cm = obj.curvemanager;
	end
    
    function c = getCurrentCurve(obj)
        if (isempty(obj.currentcurve))
            c = Curve();  %FIXME mem loss
        else
            c = obj.currentcurve;
        end
    end
    
    function unloadCurrentCurve(obj)
        obj.loadNewCurve(obj.system , obj.curvemanager);
    end
    function reloadCurveManager(obj)
         loadNewCurve(obj , obj.system, CurveManager(obj , obj.system.getDefaultDiagramPath()) )
    end
    
        %%
        function val = isUnlocked(obj)
           val = obj.unlocked; 
        end
        
        function lock(obj)
            obj.unlocked = 0;
            obj.notify('stateChanged');
        end
        
        function unlock(obj)
            obj.unlocked = 1;
            obj.notify('stateChanged');
        end
        %%
        function bool = continuerAllowed(obj)
            bool = obj.unlocked && obj.curveSelected();
        end
        function bool = numericAllowed(obj)
            bool = obj.curveSelected();
        end
        
        function bool = plotAllowed(obj)
            bool = obj.curveSelected();
        end
        function bool = starterAllowed(obj)
            bool = obj.unlocked  && obj.curveSelected() && ~strcmp(obj.currentcurve.getCurveType().getLabel(),'X');
        end
        function bool = performContinueAllowed(obj)
            bool = obj.unlocked && obj.curveSelected() && obj.currentcurve.isConfigured() && obj.branchmanager.validSelectionMade();
        end     
        function bool = performExtendAllowed(obj)
            bool = obj.unlocked && obj.curveSelected() && obj.currentcurve.isConfigured() && ~obj.currentcurve.isEmpty() && obj.currentcurve.hasFile();
        end
        function bool = curveSelected(obj)
           bool =   ~isempty(obj.system) && ~isempty(obj.curvemanager) &&  ~isempty(obj.currentcurve);
        end
        
        function bool = allowCurveSelect(obj)
           bool = ~isempty(obj.system); 
        end
        function bool = allowPointSelect(obj)
           bool = obj.curveSelected() && ~obj.currentcurve.isEmpty(); 
        end        
        
        %%
        function path = getSystemsPath(obj)
           path = obj.systemspath; 
        end
        
        function pm = getPlotManager(obj)
           pm = obj.plotmanager; 
        end
        
        %%
        function shutdown(obj)
           printc(mfilename , 'start shutdown');
           obj.sessiondata.recordState(obj);
           obj.plotmanager.destructor();
           
           obj.sessiondata.saveData();
           
           
           printc(mfilename , 'end shutdown');
            
        end
       
        function bool = getSwitch(obj , switchlabel)
           bool = obj.sessiondata.getSwitch(switchlabel); 
        end
       
    end
    
end

