classdef SessionOutput
    %SESSIONOUTPUT Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        setterlist;
        
        session;
        enabled = 1;
        
        mainhandle;
        
        pauseoption;
        SPECIAL = 1;
        ALWAYS = 2;
        NEVER = 3;
        
        currentstate = 0;
        READY = 0;
        COMPUTING = 1;
        PAUSED = 2;
        ENDED = 3;
        
        doStop;
        doPause;
        
        plotpointsperiod = 1;
        
        statushandle
        outputhandle
        
        model;
        
        plotlist = {};
    end
    
    
    methods
        
        function obj = SessionOutput(session,numvarlist)
            obj.session = session;
            obj.setterlist = struct();
            obj.mainhandle = figure('Visible','off','UserData', 0); %HACK
            obj.pauseoption = session.configmanager.getValue('suspend');
            obj.doStop = 0;
            obj.doPause = 0;
            obj.model = SessionOutputModel();
            obj.plotpointsperiod = session.configmanager.getValue('plotoutput');
            
            system = session.getSystem();
            startdata = session.getStartData();
            
            coords = system.getCoordinateList();
            params = system.getParameterList();
            
            
            %XOUT
            for i = 1:length(coords)
                numvarlist.setValue('coordinates' , coords{i} , startdata.getCoordinate(coords{i})); %init;
                handle = numvarlist.getNumHandle('coordinates', coords{i});
                obj.setterlist.xout{i} = @(val) set ( handle , 'String' , num2str(val ,'%0.8g'));
            end
            for i = 1:length(params)
                numvarlist.setValue('parameters', params{i}, startdata.getParameter(params{i})); %init
                if (startdata.getFreeParameter(params{i}))
                    handle = numvarlist.getNumHandle('parameters', params{i});
                    obj.setterlist.xout{end+1} =   @(val) set ( handle , 'String' , num2str(val ,'%0.8g'));
                end
            end
            
            %HOUT
            obj.setterlist.hout{1} = @(val) set( numvarlist.getNumHandle_s('current_stepsize') , 'String' , num2str(val ,'%0.8g'));
            obj.setterlist.hout{2} = @(val) 0;
            
            %userfunctions
             
             if (startdata.userfunctionsEnabled())
                 ufnr = startdata.getNrUserfunctions();
                 for i = 1:ufnr
                     if (startdata.isValidUserfunction(i))

                         if (startdata.isEnabledUserfunction(i))
                            ufhandle = numvarlist.getNumHandle('user_functions' , startdata.getLabelUserfunction(i));
                            
                            disp(ufhandle);
                            obj.setterlist.hout{end+1} = @(val) set ( ufhandle , 'String' , num2str(val ,'%0.8g'));
                         else
                            obj.setterlist.hout{end+1} = @(val) 0;
                         end
                     else
                         obj.setterlist.hout{end+1} = @(val) 0;
                     end

                 end
             end

        %singularities
            ignoresingularity = session.optioninterface.IgnoreSingularity;
            %calculate real ignoresingularity: %FIXME meer uitleg nodig
            if (strcmp(session.getCurveType().getLabel() , 'NS'))
                singMatrix = session.getCurveType().getSingMatrix();
                singMatrix(ignoresingularity , :) = [];
                realignoresingularity = find(~sum((8-singMatrix),1));
            else
                realignoresingularity = ignoresingularity ;
            end
            
            if (session.optioninterface.Singularities)
                list = numvarlist.getNumVarList('testfunctions');
                for i = 1:length(list)
                    if (ismember(i , realignoresingularity ))
                        %Ignore
                    else
                        if (ismember(i,ignoresingularity))
                            obj.setterlist.hout{end+1} =   @(val) 0; %pseudoignore
                        else
                            handle = numvarlist.getNumHandle('testfunctions' ,  list{i});
                            obj.setterlist.hout{end+1} =   @(val) set ( handle , 'String' , num2str(val ,'%0.8g'));
                        end
                    end
                end
            end

            
            
            
            %FOUT
            
            if(startdata.getMultipliers())
            for i = 1:length(coords)
                handle1 = numvarlist.getNumHandle('multipliers' , ['mod_' num2str(i)]);
                handle2 = numvarlist.getNumHandle('multipliers' , ['arg_' num2str(i)]);
                obj.setterlist.fout{i} = @(val) setMultiplier(handle1, handle2 , val);
            end
            else
               obj.setterlist.fout = {}; 
            end
            
            nhandle = numvarlist.getNumHandle_s('npoints');
            obj.setterlist.npoints = @(val) set( nhandle , 'String' , num2str(val ,'%0.8g'));
            
            %en nu plotting:
            obj.plotlist = session.getPlotManager().preparePlotList();
        end


        
        function output(obj,numsing,xout,s,hout,fout,ind)
        global sOutput
            

            iter = ind(end);  %waarom []?
            
            if (sOutput.enabled)
                
                 xout_len = size(xout,1);
                 hout_len = size(hout,1);
                 fout_len = size(fout,1);
                 %assert(xout_len == size(xout,1));
%                 assert(hout_len == size(hout,1));
%                 assert(fout_len == size(fout,1));

                 for i = 1:min(xout_len,length(sOutput.setterlist.xout))
                     sOutput.setterlist.xout{i}( xout(i,iter) );
                 end
                 for i = 1:min(hout_len,length(sOutput.setterlist.hout))
                     sOutput.setterlist.hout{i}( hout(i,iter) );
                 end
                 for i = 1:min(fout_len, length(sOutput.setterlist.fout))
                     sOutput.setterlist.fout{i}( fout(i,iter) );
                 end
                 sOutput.setterlist.npoints(iter);
                 
                 if (~isempty(hout))
                     for i = 1:length(obj.plotlist)
                        obj.plotlist{i}.output(obj.session, xout,s,hout,fout,ind); 
                     end
                 end
            end
        end
        
        function switchOffNumeric(obj)
        global sOutput
            for i = 1:length(sOutput.setterlist.xout)
               sOutput.setterlist.xout{i} = @(v) 0; 
            end
            for i = 1:length(sOutput.setterlist.hout)
               sOutput.setterlist.hout{i} = @(v) 0; 
            end
            for i = 1:length(sOutput.setterlist.fout)
               sOutput.setterlist.fout{i} = @(v) 0; 
            end            
            sOutput.setterlist.npoints = @(v) 0;
        end
        
        function setDuration(obj, string)
        global sOutput
            disp(['duration: ' string]);
        end
        function setStatus(obj, string)
        global sOutput
            set(obj.statushandle , 'String' ,  string);
        end
        
        function endRun(obj)
        global sOutput
            sOutput.setCurrentState(sOutput.ENDED);
            sOutput.setStatus('Finished');
        end
        
        function doStop = checkPauseResumeStop(obj, s_index , s_msg , it)
        global sOutput 
            doStop = sOutput.doStop;
            if (doStop)
                sOutput.doStop = 0;
                return;
            end %doStop = 0;
            
            if (sOutput.doPause)
                sOutput.waitForResume();
            end
            
            if (sOutput.pauseoption == sOutput.SPECIAL)
                if ((it > 2) && ( s_index == (it - 1) )) %first point excluded(it >2)
                    sOutput.pause();
                    sOutput.waitForResume(s_msg);
                end
                
            elseif (sOutput.pauseoption == sOutput.ALWAYS)
                sOutput.pause();
                if ((it > 2) && ( s_index == (it - 1) ))
                    sOutput.waitForResume();
                else
                    sOutput.waitForResume(s_msg);
                end
             end
            
        end
        
        function performStop(obj)
        global sOutput
            sOutput.doStop = 1;
            if (sOutput.currentstate == sOutput.PAUSED)
               sOutput.unpause(); 
            end
        end
        
        
        function waitForResume(obj,msg)
        global sOutput
            sOutput.setCurrentState(sOutput.PAUSED);
            
            if (nargin == 2)
                
                statusmsg = ['Paused, ' strtrim(msg)];
            else
                statusmsg = 'Paused'; 
            end
            sOutput.setStatus(statusmsg);
            waitfor(sOutput.mainhandle,'UserData',0);
            sOutput.setCurrentState(sOutput.COMPUTING);
            sOutput.setStatus('Computing ...');
            sOutput.doPause = 0;
        end
        
        
        function pause(obj)
        global sOutput
            set(sOutput.mainhandle, 'UserData' , 1);
            sOutput.doPause = 1;
        end
        
        
        function unpause(obj)
        global sOutput
            sOutput.doPause = 0;
            set(sOutput.mainhandle, 'UserData' , 0);
        end
        
        function n = getPlotPointsInterval(obj)
           n = obj.plotpointsperiod; 
        end
        
        
        function setPauseAlways(obj)
        global sOutput
            sOutput.pauseoption = sOutput.ALWAYS;
            sOutput.model.notify('settingChanged');
        end
        function setPauseSpecial(obj)
        global sOutput
            sOutput.pauseoption = sOutput.SPECIAL;
            sOutput.model.notify('settingChanged');
        end       
        function setPauseNever(obj)
        global sOutput
            sOutput.pauseoption = sOutput.NEVER;
            sOutput.model.notify('settingChanged');
        end 
        
        function  b = isPauseNever(obj)
        global sOutput
            b = sOutput.pauseoption == sOutput.NEVER;
        end
        function  b = isPauseAlways(obj)
        global sOutput
            b = sOutput.pauseoption == sOutput.ALWAYS;
        end        
        function  b = isPauseSpecial(obj)
        global sOutput
            b = sOutput.pauseoption == sOutput.SPECIAL;
        end       
        
        function setCurrentState(obj,state)
        global sOutput
            sOutput.currentstate = state;
            sOutput.model.notify('stateChanged');
        end
        function b = hasEnded(obj)
        global sOutput
            b = (sOutput.currentstate == obj.ENDED);
        end
        
        function f = getReportFunction(obj)
           f = @report;
        end
	function report(obj,varargin)
		report(varargin{:});
    end
    function setStatusHandle(obj, statushandle)
    global sOutput
       sOutput.statushandle = statushandle; 
    end
    function setOutputHandle(obj, outputhandle)
    global sOutput
       sOutput.outputhandle = outputhandle; 
    end    
    end
    methods(Static)
        function s = vector2str(varargin)
            s = ['( ' sprintf('%f ',varargin{1}) ')'];
        end
    end
end

function setMultiplier(handlemod, handlearg , val)
    set(handlemod, 'String' , num2str(abs(val) ,'%0.8g'));
    set(handlearg, 'String' , num2str((180/pi)*angle(val) ,'%0.8g'));
end

function report(varargin)
global sOutput
if (sOutput.outputhandle)
    s = sprintf(varargin{:});
    fprintf(s);
    ss = get(sOutput.outputhandle, 'String');
    set( sOutput.outputhandle , 'String' ,   [ss ; s] , 'ListboxTop' , length(ss));
end
end

